#!/usr/bin/perl

use strict;
use warnings;

use FDN::Sql::Structure;
use FDN::Common::Config;
use Getopt::Long;


sub exit_help() {
	print STDERR "\nSyntax 1: $0 [-f <file>] -c <config> [--run] [--force]\n".
		"\t<file> config file name (relative to /etc/fdn) default check-db.cfg\n".
		"\t<config> config section name describing the database\n".
		"\nSyntax 2: $0 <action> [options]\n".
		"  for action 'structure':\n".
		"\t--sql-file <file.sql>\t[+]\tAdd that file to the structure\n".
		"\t--db <name>  	\t[1]\tThe name of the database to process\n".
		"\t--force		\t\tDrop and recreate the whole databse\n".
		"\t--dry		\t\tdry-run\n".
		"\t\tdry_db:    do not create db\n".
		"\t\tdry_table: do not create/delete tables\n".
		"\t\tdry_alter: do not create/drop columns or index in tables\n".
		"\t\tdry_data:  do not create/delete/update data\n".
		"\t--run		\t\tOpposite to --dry, default\n".
		"    [1] To be provided precisely one time, mandatory\n".
		"    [+] At least one file, as many as you wish\n\n".
		"  for action 'config' or 'create':\n".
		"\t--login <login>	\t[1]\tlogin used for mysql\n".
		"\t--passwd <passwd>    \t[?]\tpassword, default to 'toto'\n".
		"\t--db <name>  	\t[1]\tname of the database\n".
		"\n".
		"  for 'config' only	\tfor 'create' only\n".
		"\t--type mysql 	\t--source host (default %)\n".
		"\t--utf8 perl  	\t\n".
		"\t--host localhost	\t\n".
		"    [1] To be provided precisely one time, mandatory\n".
		"    [?] optional\n\n";
	exit(1);
}

my $DBS = {};
my $help = 0;

if ( $ARGV[0] =~ /^-/ ) {
	my $result = GetOptions (
		'file|f=s' => sub { $DBS->{'config_file'} = $_[1] },
		'config=s' => sub { read_config(undef,$_[1],$DBS) },
		'quiet' => \&set_quiet,
		'help'   => \$help,
		'dry' => sub { switch_option('dry',1,'all') },
		'run' => sub { switch_option('dry',0,'all') },
		'force' => sub { $DBS->{'force'} = $_[1] }
		);

	if ( $help ) {
		exit_help();
	}
	if ( ! $result ) {
		print STDERR "Invalid options\n";
		exit_help();
	}

	check_db($DBS);
	exit(0);
}

my $action = shift;

if ( ! defined $action ) {
	print STDERR "No action\n\n";
	exit_help();
}

if ( $action !~ /^(structure|create|config)$/ ) {
	print STDERR "Action <$action> unexpected\n\n";
	exit_help();
}

my $result;
print "Action to perform is $action\n";
if ( $action eq 'structure' ) {
	$result = GetOptions (
		'sql-file=s{,}' => sub { $DBS->{'files'} //= []; push @{$DBS->{'files'}}, $_[1] },
		'sql-filter=s{,}' => sub { $DBS->{'filters'} //= []; push @{$DBS->{'filters'}}, $_[1] },
		'db=s' => sub { $DBS->{'name'} = $_[1] },
		'force' => sub { $DBS->{'force'} = $_[1] },
		'quiet' => \&set_quiet,
		'help' => \$help,
		'dry' => sub { switch_option('dry',1,'all') },
		'dry_db' => sub { switch_option('dry',1,'db') } ,
		'dry_table' => sub { switch_option('dry',1,'table') },
		'dry_data' => sub { switch_option('dry',1,'data') },
		'dry_alter' => sub { switch_option('dry',1,'alter') },
		'run' => sub { switch_option('dry',0,'all') },
		'run_db' => sub { switch_option('dry',0,'db') } ,
		'run_table' => sub { switch_option('dry',0,'table') },
		'run_data' => sub { switch_option('dry',0,'data') },
		'run_alter' => sub { switch_option('dry',0,'alter') },
	);
} elsif ( $action eq 'create' ) {
	$result = GetOptions (
		'help' => \$help,
		'db=s'     => sub { $DBS->{'name'}   = $_[1] },
		'login=s'  => sub { $DBS->{'login'}  = $_[1] },
		'passwd=s' => sub { $DBS->{'passwd'} = $_[1] },
		'source=s' => sub { $DBS->{'source'} = $_[1] },
		'config=s' => sub { read_config(undef,$_[1],$DBS) },
		'dry' => sub { switch_option('dry',1,'all') },
		'run' => sub { switch_option('dry',0,'all') },
	);
} elsif ( $action eq 'config' ) {
	$result = GetOptions (
		'help' => \$help,
		'db=s'     => sub { $DBS->{'name'}   = $_[1] },
		'login=s'  => sub { $DBS->{'login'}  = $_[1] },
		'passwd=s' => sub { $DBS->{'passwd'} = $_[1] },
		'utf8=s'   => sub { $DBS->{'utf8'}   = $_[1] },
		'type=s'   => sub { $DBS->{'type'}   = $_[1] },
		'host=s'   => sub { $DBS->{'host'}   = $_[1] },
		'config=s' => sub { read_config(undef,$_[1],$DBS) },
		'dry' => sub { switch_option('dry',1,'all') },
		'run' => sub { switch_option('dry',0,'all') },
	);
} else {
	print STDERR "Action <$action> not yet implemented\n\n";
	exit_help();
}

if ( ! $result ) {
	print STDERR "Unknown action $action or invalid options\n";
	exit_help;
}

if ( $help ) {
	exit_help;
}

if ( ! control_options($DBS, $action) ) {
	exit(1);
}

$subst->{'MAIN_DOM'} = Config_Key('/etc/gcmp/common.cfg','domains','main');

check_db($DBS,$action);



